/* Folder namespace */
impro.folder = {toolbar:{}};

/* init folder view */
impro.folder.init = function() {
	impro.folder.loadImages();
	
	impro.folder.initEvents();
};

/* add to editor action */
impro.folder.toolbar.add = function(li) {
	var content = null;
	var ext = li.attr('data-ext').toLowerCase();
	var id  = li.attr('data-attachment-id');
	
	var addContent = function(ct) {
		impro.editor.getTinyMCE().execCommand('mceInsertContent',false, ct);
	};
	
	/* if we add an image */
	if (['jpg','jpeg','gif','png'].indexOf(ext) !== -1) {
		var src = li.find('img').attr('src');
		content = '<img src="' + src + '" />';
		addContent(content);
	} else {	/* for any other extension, default is link, but we can easily override */
		jQuery.getJSON(impro.url + '/src/request/getattachmentdata.php?id=' + id, function(json) {	/* get all the data of the attachment */
			if (impro.folder.custom && typeof impro.folder.custom['extension_' + ext] === 'function') {	/* if user defined a custom function, use it */
				content = impro.folder.custom['extension_' + ext].call(this, json, li);
			} else {	/* if not, just make a link */
				content = ' <a href="' + json.attachment_url + '" target="_blank">' + json.post_title + '</a> ';
			}
			addContent(content);
		});
	}	
	
	return false;
};

/* edit in library */
impro.folder.toolbar.library = function(li) {
	var content = null;	
	var att_id = li.attr('data-attachment-id');

	content = impro.admin_url + 'media.php?attachment_id=' + att_id + '&action=edit';	/* build media edit link */
	
	jQuery('#vimg-toolbar a[data-action=library]').attr('href', content).attr('target', '_blank');	/* set link */
	
	return true;	/* return true, such that the href link will be used */
}

/* open */
impro.folder.toolbar.open = function(li) {
	var content = null;	
	var id = li.attr('data-attachment-id');

	jQuery.getJSON(impro.url + '/src/request/getattachmentdata.php?id=' + id, function(json) {	/* get all the data of the attachment */
		var result = window.open(json.attachment_url, 'op');
		if (!result) {
			impro.err(imageproFolderL10N.popupOpenError);
		}
	});	
	
	return false;
};

/* show link */
impro.folder.toolbar.link = function(li) {
	var content = null;	
	var id = li.attr('data-attachment-id');

	jQuery.getJSON(impro.url + '/src/request/getattachmentdata.php?id=' + id, function(json) {	/* get all the data of the attachment */
		prompt(imageproFolderL10N.attachmentLink, json.attachment_url);
	});	
	
	return false;
};

/* delete */
impro.folder.toolbar.remove = function(li) {
	if (confirm(imageproFolderL10N.deleteAttachmentConfirmation)) {
		jQuery.post(impro.url + '/src/request/deleteattachment.php', {
			id: li.attr('data-attachment-id'),
			_ajax_nonce: impro.nonce.deleteNonce
		}, function(data) {
			if (data.ok) {
				impro.info(data.data.msg);
				impro.folder.loadImages();
			} else {
				impro.err(data.data.msg);
			}
		});
	}
	return false;
};

/* init the events */
impro.folder.initEvents = function() {
	
	var timerID = 0;
	
	/* searching text event */
	jQuery('#vimg-search').keyup(function(ev) {
		if (ev.which && (/[a-zA-Z0-9 \.,;-_]/.test(String.fromCharCode(ev.which)) || ev.which === 8)) {
			clearTimeout(timerID);		
			timerID = setTimeout(function() {
				impro.folder.loadImages();
			}, 500);
		}
	});
	
	var changeFileType = function(e) {
		impro.folder.loadImages();
	};
	
	jQuery('#vimg-filetype').change(changeFileType).keyup(changeFileType);
		
	/* when a toolbar button is clicked */
	jQuery('#vimg-toolbar').click(function(ev) {
		if (!jQuery(ev.target).is('a[data-action]')) {	/* if not clicked on a link, do nothing */
			return false;
		}
		
		if (jQuery('#vimg-list').find('.impro_folder_selected').size() !== 1) {	/* if no image is selected */
			impro.err(imageproFolderL10N.noImageSelected);
			return false;
		}
		
		/* run the action on the toolbar */
		return impro.folder.toolbar[jQuery(ev.target).attr('data-action')].call(this, jQuery('#vimg-list').find('.impro_folder_selected'));
	});
	
	/* when an image is clicked */
	jQuery('#vimg-list').click(function(ev) {
		var target = jQuery(ev.target);
		
		if (ev.target === this) {	/* if clicked exactly on vimg-list, do nothing */
			return false;
		}
		
		if (!target.is('li')) {	/* if not clicked on the list item, get the list item */
			target = target.parents('li');			
		}
		
		/* select the clicked image */
		jQuery('#vimg-list li').removeClass('impro_folder_selected');
		target.addClass('impro_folder_selected');
		
		var liimg = target.find('img');
		var src = liimg.attr('src');
		
		/* if "Select another picture" was previously clicked */
		if (impro.editor.$chosen) {
			/* get the width and height of the currently selected image */
			var old_width = impro.editor.$chosen.width();
			var old_height = impro.editor.$chosen.height();
			
			/* parse the url of the new replacement image */
			var props = impro.editor.parseUrl(src);
			
			/* get the width and height of the new replacement image */
			var new_width = liimg.width();
			var new_height = liimg.height();
			var new_ratio = new_width / new_height;
			
			/* set the width and height to the old one such that proportion is kept */
			props['w'] = old_width;
			props['h'] = parseInt(old_width / new_ratio);
			
			/* resize img element if necessary */
			impro.editor.$chosen.attr('width', props['w']).attr('_mce_width', props['w']).attr('data-mce-width', props['w']);
			impro.editor.$chosen.attr('height', props['h']).attr('_mce_height', props['h']).attr('data-mce-height', props['h']);
			
			/* build the thumb and resize it */
			impro.editor.resize(impro.editor.$chosen, props );
			
			/* make advanced scale plugin forget the previous dimensions */
			impro.editor.$chosen.removeAttr('mce_advimageresize_id');
			
			/* simulate a click event to the image */
			impro.editor.showEditor(impro.editor.$chosen);

			/* clean up */
			impro.editor.$chosen = null;
			jQuery('#vimg-list').removeClass('impro_choosebg');
			jQuery.scrollTo('#titlediv', 800);
		}
		
		
	});
	
	/* this event is used for blocking the dragging of non-image files to the editor */
	jQuery('#vimg-list').mousedown(function(ev) {
		var target = jQuery(ev.target);
		
		if (target.is('img')) {	/* if began dragging an image */
			target = target.parents('li');	/* get it's li parent */
		} else {
			return false;
		}
		
		var ext = target.attr('data-ext');	/* get extension */
		
		if (ext && ['jpg', 'jpeg', 'gif', 'png'].indexOf(ext) === -1) {	/* if not image */		
			/* stop dragging */
			ev.stopPropagation();
			ev.preventDefault();
			setTimeout(function() {	/* it seems that showing an alert box immidiately makes the dragged image load. so we time out a bit */
				//impro.err('You can only drag images over the editor! For other file types, click "Add to editor".');
			}, 500);
			return false;
		}
	});
};

/* load images in the folder view */
impro.folder.loadImages = function() {
	jQuery('#vimg-list ul').load(impro.url + '/src/request/getimages.php?filetype='+jQuery('#vimg-filetype').val()+'&search='+jQuery('#vimg-search').val()+'&sort='+jQuery('#vimg-sort').val()+'&rnd='+Math.random(), function() {
		/* set up the lazy load of images on scroll*/
		impro.folder.lazyscroll();
		jQuery('#vimg-list').scroll();
	});
};

/* lazy load of images */
impro.folder.lazyscroll = function() {
	impro.folder.lazyscroll_timers = [];	
	jQuery('#vimg-list').scroll(function() {
		for(var i=0;i<impro.folder.lazyscroll_timers.length;i++) {
			clearTimeout(impro.folder.lazyscroll_timers[i]);
		}
		
		impro.folder.lazyscroll_timers.push(setTimeout(function() {
			var scrollTop = jQuery('#vimg-list').scrollTop();
			var containerTop = Math.round(jQuery('#vimg-list').offset().top);
			
			/* if there are any images in the list */
			if (jQuery('#vimg-list img:eq(0)').size() > 0) {
				var firstImageTop = Math.round(jQuery('#vimg-list img:eq(0)').offset().top);
				
				var j = 0;
				jQuery('#vimg-list img[data-lazy-src]').each(function(i) {
					var _currentImageTop = jQuery(this).offset().top;
					if (_currentImageTop - containerTop > 0) {
						
						var _el = jQuery(this);
						setTimeout(function() {
							_el.attr('src',_el.attr('data-lazy-src')).removeAttr('data-lazy-src');
						}, j++*20);
						
						if (j==26) return false;
					}
				});
			}
			
		},500));
	});
};

/* init drag and drop */
impro.folder.ddu = new impro.DragDropUpload();
impro.folder.ddu.init('vimg-list', {
	url: impro.url+'/src/request/upload.php',
	onProgress: function(e, f) {
		/* show progress */
		jQuery('#vimg-progress-wrapper').show();
		jQuery('#vimg-progress').show().width(Math.round((e.position * 100) / e.totalSize)+'%');
	},
	onComplete: function( ev ){
		/* get the target - in this case de XMLHttpRequest */
		var target = ev.target || ev.originalTarget || ev.currentTarget;
		
		/* try to parse the server response. exceptions may be thrown, or result may be null */
		var result;
		try {
			result = jQuery.parseJSON(target.responseText);
		} catch(e) {}
		if (!result) {
			impro.err("Cannot get response after file upload!");
			return;
		}
		
		/* server-side handled error */
		if (!result['ok']) {
			impro.err(result['data']['str']);
			return;
		}
		
		/* hide progress */
		jQuery('#vimg-progress-wrapper').hide();
		jQuery('#vimg-progress').hide();
		
		/* load images on the server */
		impro.folder.loadImages();
	},
	onDropFile: function(e, fn) {
		
	}
});

jQuery(function() {
	impro.folder.init();
});